#![forbid(unsafe_code)]

use std::{path::{self, Path}, thread, time::Duration};

use log;
use teloxide_core::{prelude::*, types::{Recipient, UpdateKind}};
use teloxide::Bot;
use tokio::fs;

mod gitea;
mod api;

const POLL_TIME: &str = dotenvy_macro::dotenv!("POLL_TIME", "Poll time is not set!");
const CHAT_ID: &str = dotenvy_macro::dotenv!("CHAT_ID", "Chat ID is not set!");
const TG_KEY: &str = dotenvy_macro::dotenv!("TG_KEY", "Telegram key is not set!");
const GITEA_KEY: &str = dotenvy_macro::dotenv!("GITEA_KEY", "Gitea key is not set!");
const GITEA_URL: &str = dotenvy_macro::dotenv!("GITEA_URL", "Gitea url is not set!");

#[tokio::main]
async fn main() {
    
    #[cfg(debug_assertions)]
    femme::with_level(log::LevelFilter::Debug);

    #[cfg(not(debug_assertions))] {
        femme::with_level(log::LevelFilter::Info);
        log::info!("Running in production");
    }

    log::info!("Gitea URL: {}", GITEA_URL);

    let user = api::get_user().await;
    log::info!("Logged into gitea as user \"{}\"", user.full_name);

    let repos = api::get_org_repos("Tochka".into()).await;
    log::info!("Found {} repositories", repos.len());

    let repos = api::filter_repos(repos);
    log::info!("Only  {} repositories are useful", repos.len());

    if !Path::new(".pr-cache").exists() {
        let prs = api::get_all_prs(repos.clone()).await;
        let data = serde_json::to_string_pretty(&prs).unwrap();
        fs::write(".pr-cache", data).await.unwrap();
    }

    let mut prs = serde_json::from_str::<Vec<gitea::Pull>>(String::from_utf8(fs::read(".pr-cache").await.unwrap()).unwrap().as_str()).unwrap();
    log::info!("Got {} PRs", prs.len());

    log::info!("Starting up telegram bot...");
    let bot = Bot::new(TG_KEY);
    let me = bot.get_me().await.unwrap();
    log::info!("Logged in to telegram as \"{}\"", me.first_name);
    
    let chat = bot.get_chat(Recipient::Id(ChatId(CHAT_ID.parse::<i64>().unwrap()))).await.unwrap();
    log::info!("Using chat {}", chat.title().unwrap());

    log::info!("Starting polling SCM for updates ({})", POLL_TIME);
    
    loop {
        let (new_prs, new_data) = api::get_new_prs(prs.clone(), repos.clone()).await;
        log::info!("Found {} new PRs", new_prs.len());
        thread::sleep(Duration::from_secs(5));

        if new_prs.len() != 0 {
            prs = new_data;
            let data = serde_json::to_string_pretty(&prs).unwrap();
            fs::write(".pr-cache", data).await.unwrap();

            for pr in new_prs.iter() {
                bot.send_message(chat.id, format!("New PR\n{} ({}#{}) by {}\n{}", pr.title, pr.head.repo.name, pr.number, pr.user.login, pr.url)).await.unwrap();
                bot.send_message(chat.id, "апрувните пж @bleki42 @balistiktw @x3paerz").await.unwrap();
            }
        }
    }

}
